import { ref, computed, onMounted, openBlock, createElementBlock, normalizeStyle, unref, createElementVNode, withDirectives, normalizeClass, vShow, Fragment, createVNode, Transition, withCtx, createCommentVNode, renderSlot } from "vue";
const MAIN_IMAGE_LOAD_SUCCESS = "success";
const MAIN_IMAGE_LOAD_ERROR = "error";
const DEFAULT_IMAGE_POLL_INTERVAL = 10;
const DEFAULT_IMAGE_BLUR = 20;
const DEFAULT_IMAGE_RENDERING_DELAY = 0;
const DEFAULT_IMAGE_ASPECT_RATIO = 0.5625;
const useImage = (src, {
  imagePollInterval = DEFAULT_IMAGE_POLL_INTERVAL,
  imageAspectRatio = DEFAULT_IMAGE_ASPECT_RATIO
} = {}) => {
  if (!src) {
    return;
  }
  const image = new Image();
  const naturalWidth = ref(0);
  const naturalHeight = ref(0);
  const aspectRatio = computed(() => {
    const width = naturalWidth.value;
    const height = naturalHeight.value;
    return width ? height / width : imageAspectRatio * 1;
  });
  const pollImageData = setInterval(() => {
    if (image && image.naturalWidth) {
      clearInterval(pollImageData);
      naturalWidth.value = image.naturalWidth;
      naturalHeight.value = image.naturalHeight;
    }
  }, imagePollInterval * 1);
  image.src = src;
  return {
    image,
    aspectRatio,
    naturalWidth,
    naturalHeight,
    onLoad: (fn) => image.onload = fn,
    onError: (fn) => image.onerror = fn
  };
};
const objectToArray = (map) => {
  return Object.keys(map).reduce((acc, key) => {
    return map[key] ? [...acc, key] : acc;
  }, []);
};
var ProgressiveImage_vue_vue_type_style_index_0_lang = "";
const _hoisted_1 = ["src", "alt"];
const _hoisted_2 = ["src"];
const _hoisted_3 = {
  key: 1,
  class: "v-progressive-image-slot-default"
};
const _sfc_main = {
  props: {
    src: {
      type: String,
      required: true
    },
    placeholderSrc: {
      type: String
    },
    fallbackSrc: {
      type: String
    },
    delay: {
      type: [Number, String],
      default: DEFAULT_IMAGE_RENDERING_DELAY
    },
    blur: {
      type: [Number, String],
      default: DEFAULT_IMAGE_BLUR
    },
    alt: {
      type: String
    },
    circle: {
      type: Boolean,
      default: false
    },
    objectCover: {
      type: Boolean,
      default: false
    },
    objectContain: {
      type: Boolean,
      default: false
    },
    selectNone: {
      type: Boolean,
      default: false
    },
    aspectRatio: {
      type: [Number, String],
      default: DEFAULT_IMAGE_ASPECT_RATIO
    },
    pollInterval: {
      type: [Number, String],
      default: DEFAULT_IMAGE_POLL_INTERVAL
    }
  },
  emits: [MAIN_IMAGE_LOAD_SUCCESS, MAIN_IMAGE_LOAD_ERROR],
  setup(__props, { emit }) {
    const props = __props;
    const canvasRef = ref(null);
    const mainImageRef = ref(null);
    const { aspectRatio, naturalWidth, onError, onLoad } = useImage(props.src, {
      imageAspectRatio: props.aspectRatio
    });
    const isMainImageRendered = ref(false);
    const isFallbackImageRendered = ref(false);
    const isLoading = computed(() => !isMainImageRendered.value);
    const imageClasses = computed(() => {
      return objectToArray({
        "v-progressive-image-object-cover": props.objectCover,
        "v-progressive-image-object-contain": props.objectContain,
        "v-progressive-image-select-none": props.selectNone
      });
    });
    const paddingHack = computed(() => {
      const padding = aspectRatio.value * 100;
      return {
        paddingBottom: `${padding}%`
      };
    });
    const componentMaxWidth = computed(() => {
      if (props.objectCover || props.objectContain) {
        return "auto";
      }
      if (naturalWidth.value) {
        return `${naturalWidth.value}px`;
      }
      return "100%";
    });
    const componentStyle = computed(() => {
      return {
        borderRadius: props.circle ? "50%" : 0,
        maxWidth: componentMaxWidth.value
      };
    });
    const placeholderStyle = computed(() => {
      const blurAmount = props.blur * 1;
      return {
        filter: `blur(${blurAmount}px)`
      };
    });
    const loadMainImage = () => {
      onLoad(() => {
        let canvasCtx;
        try {
          canvasCtx = canvasRef.value.getContext("2d");
          canvasCtx.drawImage(mainImageRef.value, 0, 0);
        } catch {
        }
        setTimeout(() => {
          isMainImageRendered.value = true;
          emit(MAIN_IMAGE_LOAD_SUCCESS);
        }, props.delay * 1);
      });
      onError((error) => {
        isMainImageRendered.value = true;
        isFallbackImageRendered.value = true;
        emit(MAIN_IMAGE_LOAD_ERROR, error);
      });
    };
    onMounted(() => {
      loadMainImage();
    });
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("div", {
        class: "v-progressive-image",
        style: normalizeStyle(unref(componentStyle))
      }, [
        createElementVNode("div", {
          style: normalizeStyle(unref(paddingHack))
        }, [
          withDirectives(createElementVNode("img", {
            class: normalizeClass(["v-progressive-image-main", unref(imageClasses)]),
            ref_key: "mainImageRef",
            ref: mainImageRef,
            src: isFallbackImageRendered.value ? __props.fallbackSrc : __props.src,
            alt: __props.alt
          }, null, 10, _hoisted_1), [
            [vShow, isMainImageRendered.value]
          ]),
          __props.placeholderSrc ? (openBlock(), createElementBlock(Fragment, { key: 0 }, [
            createVNode(Transition, {
              appear: "",
              "leave-class": "v-progressive-image-fade-leave",
              "leave-to-class": "v-progressive-image-fade-leave-to",
              "leave-active-class": "v-progressive-image-fade-leave-active"
            }, {
              default: withCtx(() => [
                unref(isLoading) ? (openBlock(), createElementBlock("img", {
                  key: 0,
                  class: "v-progressive-image-placeholder",
                  style: normalizeStyle(unref(placeholderStyle)),
                  src: __props.placeholderSrc
                }, null, 12, _hoisted_2)) : createCommentVNode("", true)
              ]),
              _: 1
            }),
            unref(isLoading) ? (openBlock(), createElementBlock("canvas", {
              key: 0,
              hidden: "",
              width: "1",
              height: "1",
              ref_key: "canvasRef",
              ref: canvasRef
            }, null, 512)) : createCommentVNode("", true)
          ], 64)) : createCommentVNode("", true),
          _ctx.$slots.default ? (openBlock(), createElementBlock("div", _hoisted_3, [
            renderSlot(_ctx.$slots, "default", { isLoading: unref(isLoading) })
          ])) : createCommentVNode("", true)
        ], 4)
      ], 4);
    };
  }
};
const install = (app) => {
  app.component("ProgressiveImage", _sfc_main);
};
export { _sfc_main as ProgressiveImage, install as default };
